/*
 ATtiny25 Tiny Temperature Logger - Reader
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/11/2012
 Last Modified: 3/02/2013
 
 A reader for the ATtiny25 Tiny Temperature Logger used to extract data and re-program the delay time
 We use an ATtiny85 with V-USB to read from the external I2C EEPROM.
 
 Remember to change your fuse bits so the ATtiny85 uses 16MHz PLL and divide clock by 8
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0x61:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m 
 
 */

// ATtiny85 Pin map
//
//             +-\/-+
// Reset PB5  1|o   |8  VCC
//       PB3  2|    |7  PB2
//       PB4  3|    |6  PB1
//       GND  4|    |5  PB0
//             +----+

#define usbminusPin PB1 // USB D-
#define TWI_SDA_PIN PB4
#define TWI_SCL_PIN PB0
#define delayTimelocation 65532
#define EEPROM_ADDR 0x50 // I2C EEPROM address

#define F_CPU 16500000 // If this is changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "usbconfig.h"
#include "setup.c"

// USB transfer variables
#define USB_TRANSFER 1
#define USB_DELAYTIME 2
uint32_t delayTime = 0;
uint16_t readCount = 0;
uint16_t endCount = delayTimelocation - 1;
byte sentDelaytime = false;
byte countDigits = 0;

static uchar replyBuf[16];
static uchar *nextDigit;
static uchar tempbuffer[16];
static uchar *nextDigittemp;
static uchar dataReceived = 0, dataLength = 0; // for USB_DELAYTIME

// Convert a number to ASCII and add it to the replyBuf variable (it will only between 0 to 125)
void numberToascii(int number) {
	uchar diviser = 100;
	if ((number / diviser) != 0) {
		*++nextDigit = (number / diviser) + 48;
		*++nextDigit = ((number % diviser) / 10) + 48;
	}
	else if ((number / 10) != 0) {
		*++nextDigit = (number / 10) + 48;
	}
	*++nextDigit = (number % 10) + 48;
}

// Convert a number to ASCII values and add it to the nextDigittemp variable
void numberToasciibuffer(uint32_t number) {
	uchar digit;
	do {
		countDigits++;
		digit = number % 10;
		number /= 10;
		if (digit == 0) {
			*--nextDigittemp = 0 + 48;
		}
		else {
			*--nextDigittemp = digit + 48;
		}
	} while (number != 0);
}

// Send the delay time used when logging
void sendDelaytime(void) {
	// Re-construct 32bit value from EEPROM
	byte firstByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation);
	byte secondByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+1);
	byte thirdByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+2);
	delayTime = (uint32_t) ((uint32_t) firstByte << 16) | (secondByte << 8) | thirdByte; // Combine 3 bytes
	
	// Convert the number to ASCII and place it in a temporary buffer
	nextDigittemp = &tempbuffer[sizeof(tempbuffer)];
	countDigits = 0;
	numberToasciibuffer(delayTime);
	
	// Transfer that temporary buffer to the reply buffer
	nextDigit = &replyBuf[0];
	memcpy(nextDigit, nextDigittemp, countDigits);
	nextDigit += countDigits;
	memcpy(nextDigit, "\r\n\0", 3);
}

// Send the next digit from the EEPROM address
byte sendnextDigit(uint16_t eepromAddress) {	
	// Re-construct 10bit value (0 - 1024) from high and low byte from EEPROM
	byte highByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, eepromAddress);
	byte lowByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, eepromAddress+1);
	
	unsigned int combinedValue = (highByte << 8) | lowByte; // Combine high and low byte
	
	if (combinedValue == 65278) { // 254 high and 254 low byte, end
		return 1;
	}
	
	double calculateDouble = thermistorTemp(combinedValue);
	
	// Check if value is less than 0, if so convert it to positive number and include the negative sign at the end
	boolean isNegative = false;
	if (calculateDouble < 0) {
		calculateDouble = calculateDouble - (calculateDouble * 2);
		isNegative = true;
	}
	
	// Split the value into 2 parts
	int beforeDecimalpoint = (int) calculateDouble; // Before the decimal point
	int afterDecimalpoint = (int) ((calculateDouble - beforeDecimalpoint) * 100); // After the decimal point
	
	// Round the decimal point
	afterDecimalpoint = floor(((calculateDouble - beforeDecimalpoint) * 10) + 0.5);
	if (afterDecimalpoint == 10) { // Round up
		beforeDecimalpoint++;
		afterDecimalpoint = 0;
	}
	
	// Negative
	if (isNegative == true) {
		*++nextDigit = 45;
	}
	
	// First part before the dot
	numberToascii(beforeDecimalpoint);
	
	// Second part after the dot
	if (afterDecimalpoint != 0) {
		*++nextDigit = 46; // Dot
		
		numberToascii(afterDecimalpoint);
	}
	
	memcpy(++nextDigit, "\r\n\0", 3);
	
	return 0;
}

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TRANSFER) { // custom command is in the bRequest field
		nextDigit = &replyBuf[-1];
		if (sentDelaytime == false) { // Firstly print the logging delay used
			sendDelaytime();
			sentDelaytime = true;
		}
		else if (readCount <= endCount) {
			int checkforZero = sendnextDigit(readCount);
			readCount += 2;
			
			if (checkforZero == true) { // End
				nextDigit = &replyBuf[0];
				memcpy(nextDigit, "END\r\n\0", 6);
				
				// Reset variables
				readCount = 0;
				sentDelaytime = false;
			}
		}
		else { // End if we reached the end of the EEPROM
			nextDigit = &replyBuf[0];
			memcpy(nextDigit, "END\r\n\0", 6);
			
			// Reset variables
			readCount = 0;
			sentDelaytime = false;
		}
		
		usbMsgPtr = replyBuf;
		
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_DELAYTIME) { // Re-program the delay time
		dataLength = (uchar)rq->wLength.word;
		dataReceived = 0;
		
		if (dataLength > sizeof(replyBuf)) { // Limit to buffer size
			dataLength = sizeof(replyBuf);
		}
		
		return USB_NO_MSG; // usbFunctionWrite will be called now
	}
	
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {
	uchar i;
	
	for (i = 0; dataReceived < dataLength && i < len; i++, dataReceived++) {
		replyBuf[dataReceived] = data[i];
	}
	
	uint32_t delayTimeProgram = atol((char*) replyBuf);
	
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation, (delayTimeProgram >> 16)); // First byte
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation+1, (delayTimeProgram >> 8)); // Second byte
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation+2, (delayTimeProgram & 0xFF)); // Third byte
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

int main (void) {
	setup();
	
	while(1) {
		startUSB();
		SoftI2cMasterInit();
		while (1) {
			usbPoll(); // Keep polling USB until device is unplugged
		}
	}
	
	return 0;
}
