/*
 ATtiny25 Tiny Temperature Logger
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/11/2012
 Last Modified: 3/02/2013
 
 Used to log temperature every x seconds/minutes/hours which is saved to an external I2C EEPROM.
 You can transfer the data via USB to your computer using the a25ttl_transfer program and program the
 delay time by running the a25ttl_programdelay program.
  
 */

// ATtiny25 Pin map
//
//             +-\/-+
// Reset PB5  1|o   |8  VCC
//       PB3  2|    |7  PB2
//       PB4  3|    |6  PB1
//       GND  4|    |5  PB0
//             +----+

int analogPin = 3; // Analog input 3
int analogSCLPin = 1; // TWI_SCL_PIN
#define tempPin PB4 // Output
#define TWI_SDA_PIN PB1
#define TWI_SCL_PIN PB2
#define delayTimelocation 65532 // 4 bytes less than the 512Kbit EEPROM size
#define EEPROM_ADDR 0x50 // I2C EEPROM address

#define F_CPU 1000000 // 1 MHz clock

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <string.h>
#include "setup.c"

uint16_t dataCount = 0;
uint32_t delayTime = 0;

int main (void) {
	setup();
	
	// Read delay time
	byte firstByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation);
	byte secondByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+1);
	byte thirdByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+2);
	delayTime = (uint32_t) ((uint32_t) firstByte << 16) | (secondByte << 8) | thirdByte; // Combine 3 bytes
	if (delayTime == blankEeprom2byte) { // Unprogrammed delay time?
		delayTime = 75; // Set 5 minutes as the default delay time
	}
	
	// Break down the delay time in 8, 4, 2, 1 seconds delay
	uint32_t eightSecondsleep = delayTime / 8;
	delayTime = delayTime % 8;
	byte fourSecondsleep = delayTime / 4;
	delayTime = delayTime % 4;
	byte twoSecondsleep = delayTime / 2;
	delayTime = delayTime % 2;
	byte oneSecondsleep = delayTime / 1;
	delayTime = delayTime % 1;
	
	while(1) {
		// Log Temperature
		PORTB |= (1<<tempPin); // Turn on thermistor
		_delay_us(50); // Wait a little while before reading thermistor
		
		int calculatedValue = analogRead(analogPin); // Store the reading
		PORTB &= ~(1<<tempPin); // Turn off
		
		// Write to EEPROM (don't write to where the delay time is stored)
		if (dataCount <= delayTimelocation - 1) {
			soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, (calculatedValue >> 8));
			dataCount++;
			soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, (calculatedValue & 0xFF));
			dataCount++;
			if (dataCount <= delayTimelocation - 1) {
				soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, 254);
				soft_i2c_eeprom_write_byte(EEPROM_ADDR, (dataCount+1), 254);
			}
		}
		else {
			// Sleep forever since the EEPROM is full
			system_sleep();
		}
		
		// Wait the delay time
		uint32_t waitCounter = 0;
		while (waitCounter != eightSecondsleep) {
			watchdogSleep(T8S);
			waitCounter++;
		}
		waitCounter = 0;
		while (waitCounter != fourSecondsleep) {
			watchdogSleep(T4S);
			waitCounter++;
		}
		waitCounter = 0;
		while (waitCounter != twoSecondsleep) {
			watchdogSleep(T2S);
			waitCounter++;
		}
		waitCounter = 0;
		while (waitCounter != oneSecondsleep) {
			watchdogSleep(T1S);
			waitCounter++;
		}
	}
	
	return 0;
}

ISR(WDT_vect) { }

ISR(ADC_vect) { }
